/***************************************************************************
Name: JSView
Description: View the source code of external js and css files
Author: Ron Beckman

Copyright (C) 2008  Ron Beckman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

Free Software Foundation, Inc.
51 Franklin Street
Fifth Floor
Boston, MA  02110-1301
USA
***************************************************************************/


var jsvutils = {

	addCommas: function(numOld) {
		var numNew = parseInt(numOld);
		numNew = numNew.toString();
		var rgx = /(\d+)(\d{3})/;
		while (rgx.test(numNew)) {
			numNew = numNew.replace(rgx, '$1' + ',' + '$2');
		}
		return numNew;
	},

	convertFileToURI: function(file) {
		return Components.classes["@mozilla.org/network/protocol;1?name=file"].
		createInstance(Components.interfaces.nsIFileProtocolHandler).
		newFileURI(file);
	},

	convertFileToUrl: function(file) {
		return Components.classes["@mozilla.org/network/protocol;1?name=file"].
		createInstance(Components.interfaces.nsIFileProtocolHandler).
		getURLSpecFromFile(file);
	},

	copyToClipboard: function(txt) {
		var clip = this.getClipboard();
		clip.copyString(txt);
	},

	getAllWindows: function(name) {
		return Components.classes["@mozilla.org/appshell/window-mediator;1"].
		getService(Components.interfaces.nsIWindowMediator).
		getEnumerator(name);
	},

	getAppInfo: function() {
		return Components.classes['@mozilla.org/xre/app-info;1'].
		createInstance(Components.interfaces.nsIXULAppInfo);
	},

	getBundleService: function() {
		return Components.classes['@mozilla.org/intl/stringbundle;1'].
		getService(Components.interfaces.nsIStringBundleService);
	},

	getCacheService: function() {
		return Components.classes["@mozilla.org/network/cache-service;1"].
		getService(Components.interfaces.nsICacheService).
		createSession("HTTP", 0, true);
	},

	getClipboard: function() {
		return Components.classes["@mozilla.org/widget/clipboardhelper;1"].
		getService(Components.interfaces.nsIClipboardHelper);
	},

	getExternalCSS: function(win,embd,srt,fdup,curis,fuft,fwl,fcu) {
		var sheets,loc,uri;
		var styleUrls = [];
		var prefs = this.getPrefs();
		var io = this.getIOService();

		//get a reference to all dom windows in current page
		var wins = this.getFrames(win);

		for (var i=0; i<wins.length; ++i) {
			sheets = wins[i].document.styleSheets;
			for (var j=0; j<sheets.length; ++j) {
				if (sheets[j].ownerNode &&
				    sheets[j].ownerNode instanceof Components.interfaces.nsIDOMHTMLStyleElement) {
					//save embedded stylesheets for page info window
					if (embd) styleUrls.push(sheets[j].ownerNode);
				} else if (sheets[j].href) {
					//we need to make sure all anchors are removed from the urls
					styleUrls.push(sheets[j].href.replace(/#.*$/,''));
				}
				styleUrls = styleUrls.concat(this.getImportedCSS(sheets[j]));
			}
		}

		//we sort the urls to ensure that chrome urls appear at top of menupopup
		if (srt) styleUrls.sort();

		//check for duplicate entries and remove them
		if (fdup) styleUrls = this.removeDuplicates(styleUrls);

		//we create the uris here instead of the previous loop since the urls
		//need to be sorted and duplicates removed before the uris can be created
		if (curis) {
			for (var i=0; i<styleUrls.length; ++i) {
				try {
					uri = io.newURI(styleUrls[i],null,null);
					uri.QueryInterface(Components.interfaces.nsIURL);
					styleUrls[i] = uri;
				} catch(e) {
					styleUrls.splice(i,1);
					--i;
				}
			}
		}

		//filter the styleUrls array for unwanted file types
		if (curis && fuft) {
			var exts = prefs.getCharPref('unwantedFileTypes');
			if (exts) {
				styleUrls = styleUrls.filter(function(value) {
					var re = new RegExp('\\b'+value.fileExtension+'\\b');
					return !value.fileExtension || exts.search(re) == -1;
				});
			}
		}

		//filter the styleUrls array for any urls that match window locations
		if (fwl) {
			for (var i=0; i<wins.length; ++i) {
				loc = wins[i].location.href.replace(/#.*$/,'');
				if (curis) {
					styleUrls = styleUrls.filter(function(value) { return value.spec != loc; });
				} else {
					styleUrls = styleUrls.filter(function(value) { return value != loc; });
				}
			}
		}

		//filter the styleUrls array for chrome urls
		if (curis && fcu) {
			if (!prefs.getBoolPref('showChromeUrls')) {
				styleUrls = styleUrls.filter(function(value) {
					return value.scheme != 'chrome' && value.scheme != 'resource';
				});
			}
		}

		return styleUrls;
	},

	getExternalJS: function(win,embd,srt,fdup,curis,fuft,fwl,fcu) {
		var scriptUrls = [];
		var prefs = this.getPrefs();
		var io = this.getIOService();
		var scripts,loc,source,srcAttr,uri;

		//get a reference to all dom windows in current page
		var wins = this.getFrames(win);

		for (var i=0; i<wins.length; ++i) {
			scripts = wins[i].document.getElementsByTagName('script');
			for (var j=0; j<scripts.length; ++j) {
				if (scripts[j].hasAttribute('src')) {
					//use getAttribute for xul script elements
					//use .src for html script elements
					srcAttr = scripts[j].getAttribute('src');
					source = (srcAttr.search(/^chrome\:|^resource\:/) > -1) ? srcAttr : scripts[j].src;
					if (source) {
						//we need to make sure all anchors are removed from the urls
						scriptUrls.push(source.replace(/#.*$/,''));
					}
				} else if (embd) { //save embedded scripts for page info window
					scriptUrls.push(scripts[j]);
				}
			}
		}

		//we sort the urls to ensure that chrome urls appear at top of menupopup
		if (srt) scriptUrls.sort();

		//check for duplicate entries and remove them
		if (fdup) scriptUrls = this.removeDuplicates(scriptUrls);

		//we create the uris here instead of the previous loop since the urls
		//need to be sorted and duplicates removed before the uris can be created
		if (curis) {
			for (var i=0; i<scriptUrls.length; ++i) {
				try {
					uri = io.newURI(scriptUrls[i],null,null);
					uri.QueryInterface(Components.interfaces.nsIURL);
					scriptUrls[i] = uri;
				} catch(e) {
					scriptUrls.splice(i,1);
					--i;
				}
			}
		}

		//filter the scriptUrls array for unwanted file types
		if (curis && fuft) {
			var exts = prefs.getCharPref('unwantedFileTypes');
			if (exts) {
				scriptUrls = scriptUrls.filter(function(value) {
					var re = new RegExp('\\b'+value.fileExtension+'\\b');
					return !value.fileExtension || exts.search(re) == -1;
				});
			}
		}

		//filter the scriptUrls array for any urls that match window locations
		if (curis && fwl) {
			for (var i=0; i<wins.length; ++i) {
				loc = wins[i].location.href.replace(/#.*$/,'');
				scriptUrls = scriptUrls.filter(function(value) { return value.spec != loc; });
			}
		}

		//filter the scriptUrls array for chrome urls
		if (curis && fcu) {
			if (!prefs.getBoolPref('showChromeUrls')) {
				scriptUrls = scriptUrls.filter(function(value) {
					return value.scheme != 'chrome' && value.scheme != 'resource';
				});
			}
		}

		return scriptUrls;
	},

	getFileOutputStream: function() {
		return Components.classes["@mozilla.org/network/file-output-stream;1"].
		createInstance(Components.interfaces.nsIFileOutputStream);
	},

	getFrames: function(win) {
		if (!win) return [];

		var frms = [win];

		for (var i=0; i<frms.length; ++i) {
			for (var j=0; j<frms[i].frames.length; ++j) {
				frms.push(frms[i].frames[j]);
			}
			//we break after reaching 50 to avoid a possible endless loop from
			//occurring if the website has coded frames within frames.........
			if (i >= 50) break;
		}
		return frms;
	},

	getImportedCSS: function(css) {
		var urls = [];
		var sheets = [css];

		for (var i=0; i<sheets.length; ++i) {
			for (var j=0; j<sheets[i].cssRules.length; ++j) {
				if (sheets[i].cssRules[j].type != 3) break;

				if (sheets[i].cssRules[j].styleSheet &&
				    sheets[i].cssRules[j].styleSheet.href) {
					//we need to make sure all anchors are removed from the urls
					urls.push(sheets[i].cssRules[j].styleSheet.href.replace(/#.*$/,''));
					sheets.push(sheets[i].cssRules[j].styleSheet);
				}
			}
			//we break after reaching 50 to avoid a possible endless loop from
			//occurring if a stylesheet tries to import itself over and over...
			if (i >= 50) break;
		}
		return urls;
	},

	getIOService: function(url) {
		return Components.classes["@mozilla.org/network/io-service;1"].
		getService(Components.interfaces.nsIIOService);
	},

	getJarURI: function(uri) {
		return Components.classes['@mozilla.org/chrome/chrome-registry;1'].
		getService(Components.interfaces.nsIChromeRegistry).
		convertChromeURL(uri);
	},

	getObserver: function() {
		return Components.classes['@mozilla.org/observer-service;1'].
		getService(Components.interfaces.nsIObserverService);
	},

	getPrefs: function() {
		return Components.classes["@mozilla.org/preferences-service;1"].
		getService(Components.interfaces.nsIPrefService).
		getBranch("jsview.");
	},

	getStr: function(name) {
		return document.getElementById("jsview-bundle").getString(name);
	},

	getTempDirectory: function() {
		return Components.classes["@mozilla.org/file/directory_service;1"].
		getService(Components.interfaces.nsIProperties).
		get("TmpD", Components.interfaces.nsIFile);
	},

	getURI: function(url) {
		return Components.classes["@mozilla.org/network/io-service;1"].
		getService(Components.interfaces.nsIIOService).
		newURI(url,null,null);
	},

	getWin: function(name) {
		return Components.classes["@mozilla.org/appshell/window-mediator;1"].
		getService(Components.interfaces.nsIWindowMediator).
		getMostRecentWindow(name);
	},

	getZipReaderForFile: function(zipFile) {
		try {
			var zipReader = Components.classes["@mozilla.org/libjar/zip-reader;1"].
			createInstance(Components.interfaces.nsIZipReader);

			if ('init' in zipReader) {
				zipReader.init(zipFile);
				zipReader.open();
			} else {
				zipReader.open(zipFile);
			}
		} catch (e) {
			zipReader.close();
			throw e;
		}
		return zipReader;
	},

	promptBehavior: function(ev,menu,url) {
		if (ev.which > 1) {
			ev.preventDefault();
			menu.hidePopup();
			//check for existence of window
			var win = this.getWin('jsview:behavior');
			if (win) {
				//if exists then bring to front
				win.focus();
				//update url in text box in behavior window
				var os = this.getObserver();
				os.notifyObservers(null,'jsview-url-changed',url);
			} else {
				//if doesn't exist then open a new window
				var brows = this.getAppInfo();
				var win = this.getWin((brows.name == 'Songbird') ? 'Songbird:Main' : 'navigator:browser');
				win.openDialog("chrome://jsview/content/jsviewBehavior.xul", "_blank", "chrome,centerscreen,resizable=yes", url);
			}
		}
	},

	removeDuplicates: function(arrOld) {
		var arrNew = [];
		for (var i=0; i<arrOld.length; ++i) {
			if (arrNew.indexOf(arrOld[i]) == -1) {
				arrNew.push(arrOld[i]);
			}
		}
		return arrNew;
	},

	urlToFile: function(url) {
		return Components.classes['@mozilla.org/network/protocol;1?name=file'].
		createInstance(Components.interfaces.nsIFileProtocolHandler).
		getFileFromURLSpec(url);
	},

	viewAll: function(type,win) {
		var prefs = this.getPrefs();
		var bhvr = prefs.getIntPref('menuItemsBehavior');

		var uris = (type == 'js') ? this.getExternalJS(win,false,true,true,true,true,true,true) : this.getExternalCSS(win,false,true,true,true,true,true,true);

		for (var i=0; i<uris.length; ++i) {
			setTimeout("jsvutils.viewSource('"+uris[i].spec+"','"+bhvr+"')",i*500);
		}
	},

	viewSource: function(url,type) {
		//type: 0 = new win, 1 = new tab blur, 2 = new tab focus, 3 = external editor
		//type: 4 = menuItemsBehavior preference

		if (type == 4) {
			var prefs = this.getPrefs();
			type = prefs.getIntPref('menuItemsBehavior');
		}

		var brows = this.getAppInfo();
		var win = this.getWin((brows.name == 'Songbird') ? 'Songbird:Main' : 'navigator:browser');
		if (type == 0) {
			if (brows.name == 'SeaMonkey') {
				this.viewSourceSM(url);
			} else {
				this.viewSourceFF(url);
			}
		} else if (type == 1) {
			url = 'view-source:' + url;
			win.getBrowser().addTab(url);
		} else if (type == 2) {
			url = 'view-source:' + url;
			var newTab = win.getBrowser().addTab(url);
			win.getBrowser().selectedTab = newTab;
		} else if (type == 3) {
			jsviewExternal.openInExternalEditor(url,null,null);
		}
	},

	viewSourceFF: function(url) {
		var brows = this.getAppInfo();
		var win = this.getWin((brows.name == 'Songbird') ? 'Songbird:Main' : 'navigator:browser');
		win.openDialog("chrome://global/content/viewSource.xul", "_blank", "all,dialog=no", url, null, null);
	},

	viewSourceSM: function(url) {
		var brows = this.getAppInfo();
		var win = this.getWin((brows.name == 'Songbird') ? 'Songbird:Main' : 'navigator:browser');
		win.openDialog("chrome://navigator/content/viewSource.xul", "_blank", "all,dialog=no", url, null, null);
	}
};
