/***************************************************************************
Name: JSView
Description: View the source code of external js and css files
Author: Ron Beckman

Copyright (C) 2008  Ron Beckman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

Free Software Foundation, Inc.
51 Franklin Street
Fifth Floor
Boston, MA  02110-1301
USA
***************************************************************************/


window.addEventListener("load", jsviewViewSourceLoad, false);

function jsviewViewSourceLoad() {
	jsviewViewSource.init();
}

window.addEventListener("unload", jsviewViewSourceUnload, false);

function jsviewViewSourceUnload() {
	jsviewViewSource.exit();
}

var jsviewPrefObserver = {

	prefs: null,

	init: function() {
		this.prefs = this.getPrefs();
		this.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
		this.prefs.addObserver("", this, false);
	},

	uninit: function() {
		this.prefs.removeObserver("", this);
	},

	getPrefs: function() {
		return Components.classes["@mozilla.org/preferences-service;1"].
		getService(Components.interfaces.nsIPrefService).
		getBranch("jsview.");
	},

	observe: function(subject, topic, data) {
		if (topic != "nsPref:changed") return false;

		switch(data) {
			case "hideStatus":
			   jsviewViewSource.hideStatus();
			   break;

			case "hideView":
			   jsviewViewSource.hideView();
			   break;
		}
		return false;
	}
};

var jsviewViewSource = {

	_win: null,

	cancelContextMenu: function() {
		var ctxt = document.getElementById('jsview-menupopup');
		ctxt.hidePopup();
	},

	init: function() {
		this._win = this.getContentWin();

		//add observer to monitor jsview prefs
		jsviewPrefObserver.init();

		//show-hide statusbar icon
		this.hideStatus();

		//show-hide view menu item
		this.hideView();

		//set statusbar icon
		this.load();
	},

	exit: function() {
		//remove pref observer
		jsviewPrefObserver.uninit();
	},

	hideStatus: function() {
		var sts = document.getElementById('jsview-status');
		if (!sts) return false;

		var prefs = jsvutils.getPrefs();
		sts.setAttribute('hidden',prefs.getBoolPref('hideStatus'));
		return false;
	},

	hideView: function() {
		var viw = document.getElementById('jsview-view');
		if (!viw) return false;

		var brows = jsvutils.getAppInfo();
		var MVP = document.getElementById((brows.name == 'SeaMonkey') ? 'menu_View_Popup' : 'viewmenu-popup');
		if (!MVP) return false;

		var prefs = jsvutils.getPrefs();
		var hv = prefs.getBoolPref('hideView');
		viw.setAttribute('hidden',hv);

		if (hv) {
			MVP.removeEventListener('popupshowing', jsviewViewSource.showViewPopup, false);
		} else {
			MVP.addEventListener('popupshowing', jsviewViewSource.showViewPopup, false);
		}
		return false;
	},

	load: function() {
		var status = document.getElementById('jsview-status');
		if (!status) return false;

		var icon,hidden;
		var js = jsvutils.getExternalJS(this._win,false,true,true,true,true,true,true);
		var css = jsvutils.getExternalCSS(this._win,false,true,true,true,true,true,true);

		if (js.length > 0 && css.length > 0) {
			icon = 'both';
			hidden = 'false';
		} else if (js.length > 0) {
			icon = 'js';
			hidden = 'false';
		} else if (css.length > 0) {
			icon = 'css';
			hidden = 'false';
		} else {
			icon = 'none';
			hidden = 'true';
		}

		var label = jsvutils.getStr('jsview.JS') +
				' (' + js.length + ') ' +
				jsvutils.getStr('jsview.CSS') +
				' (' + css.length + ')';

		if (status) {
			status.setAttribute('jsview',icon);
			status.setAttribute('tooltiptext',label);
		}
		return false;
	},

	showViewPopup: function() {
		var view = document.getElementById('jsview-view');
		if (!view) return false;

		var icon,hidden;
		var js = jsvutils.getExternalJS(jsviewViewSource._win,false,true,true,true,true,true,true);
		var css = jsvutils.getExternalCSS(jsviewViewSource._win,false,true,true,true,true,true,true);

		if (js.length > 0 && css.length > 0) {
			icon = 'both';
			hidden = 'false';
		} else if (js.length > 0) {
			icon = 'js';
			hidden = 'false';
		} else if (css.length > 0) {
			icon = 'css';
			hidden = 'false';
		} else {
			icon = 'none';
			hidden = 'true';
		}

		var label = jsvutils.getStr('jsview.JS') +
				' (' + js.length + ') ' +
				jsvutils.getStr('jsview.CSS') +
				' (' + css.length + ')';

		view.setAttribute('jsview',icon);
		view.setAttribute('hidden',hidden);
		view.setAttribute('tooltiptext',label);
		view.setAttribute('label',label);
		return false;
	},

	getContentWin: function() {
		var tab = null;
		var win,brows,loc;
		var url = window.arguments[0].replace(/#.*$/,'');

		var app = jsvutils.getAppInfo();
		var wins = jsvutils.getAllWindows((app.name == 'Songbird') ? 'Songbird:Main' : 'navigator:browser');

		while (!tab && wins.hasMoreElements()) {
			win = wins.getNext();
			brows = win.getBrowser().browsers;
			for (var i=0; i<brows.length; ++i) {
				loc = brows[i].contentWindow.location.href.replace(/#.*$/,'');
				if (loc == url) {
					tab = brows[i].contentWindow;
					break;
				}
			}
		}
		return tab;
	},

	makeElement: function(attrs) {
		var elem = document.createElement(attrs[0]);
		for (var i=1; i<attrs.length; ++i) {
			elem.setAttribute(attrs[i][0],attrs[i][1]);
		}
		return elem;
	},

	populateMenu: function(menu) {
		while (menu.firstChild) {
			menu.removeChild(menu.firstChild);
		}

		var uri,attrs,fname,cname,size,entry,cacheKey;
		var prefs = jsvutils.getPrefs();
		var js = jsvutils.getExternalJS(this._win,false,true,true,true,true,true,true);
		var css = jsvutils.getExternalCSS(this._win,false,true,true,true,true,true,true);
		var cache = jsvutils.getCacheService();
		var filesize = prefs.getBoolPref('fileSizeInKB');

		//get string translations
		var done = jsvutils.getStr('jsview.Done');
		var jsnf = jsvutils.getStr('jsview.JsNF');
		var cssnf = jsvutils.getStr('jsview.CssNF');
		var noname = jsvutils.getStr('jsview.NoName');
		var kb = jsvutils.getStr('jsview.KB');
		var bytes = jsvutils.getStr('jsview.Bytes');
		var jsall = jsvutils.getStr('jsview.ViewAllJs');
		var cssall = jsvutils.getStr('jsview.ViewAllCss');

		if (js.length < 1) {
			attrs = ['menuitem',
				  ['label',jsnf],
				  ['tooltiptext',jsnf],
				  ['disabled','true'],
				  ['class','menuitem-iconic jsview-none']];
			menu.appendChild(this.makeElement(attrs));
		} else {
			attrs = ['menuitem',
				  ['label',jsall],
				  ['tooltiptext',jsall],
				  ['oncommand',"jsvutils.viewAll('js',jsviewViewSource._win);"],
				  ['class','menuitem-iconic jsview-js jsview-boldNormal']];
			menu.appendChild(this.makeElement(attrs));
		}

		for (var i=0; i<js.length; ++i) {
			//get filename from url
			fname = (js[i].fileName) ? js[i].fileName : noname;

			//add italic for chrome urls
			cname = (js[i].scheme.search(/^chrome$|^resource$/) > -1) ? 'jsview-js jsview-italic' : 'jsview-js';

			//get filesize for url if cached
			cacheKey = js[i].spec.replace(/#.*$/,'');
			try {
				entry = cache.openCacheEntry(cacheKey, Components.interfaces.nsICache.ACCESS_READ, false);
				if (filesize) {
					size = ' (' + (entry.dataSize / 1024).toFixed(2) + ' ' + kb + ')';
				} else {
					size = ' (' + jsvutils.addCommas(entry.dataSize) + ' ' + bytes + ')';
				}
			} catch(e) {
				size = '';
			}

			attrs = ['menuitem',
				  ['label',fname + size],
				  ['value',js[i].spec],
				  ['statustext',js[i].spec],
				  ['tooltiptext',js[i].spec],
				  ['oncommand','jsvutils.viewSource(this.value,4);'],
				  ['onclick','jsvutils.promptBehavior(event,this.parentNode,this.value);'],
				  ['onmouseover','jsviewViewSource.showStatusText(this.value);'],
				  ['onmouseout',"jsviewViewSource.showStatusText('" + done + "');"],
				  ['class','menuitem-iconic ' + cname]];
			menu.appendChild(this.makeElement(attrs));
		}

		menu.appendChild(this.makeElement(['menuseparator']));

		if (css.length < 1) {
			attrs = ['menuitem',
				  ['label',cssnf],
				  ['tooltiptext',cssnf],
				  ['disabled','true'],
				  ['class','menuitem-iconic jsview-none']];
			menu.appendChild(this.makeElement(attrs));
		} else {
			attrs = ['menuitem',
				  ['label',cssall],
				  ['tooltiptext',cssall],
				  ['oncommand',"jsvutils.viewAll('css',jsviewViewSource._win);"],
				  ['class','menuitem-iconic jsview-css jsview-boldNormal']];
			menu.appendChild(this.makeElement(attrs));
		}

		for (var i=0; i<css.length; ++i) {
			//get filename from url
			fname = (css[i].fileName) ? css[i].fileName : noname;

			//add italic for chrome urls
			cname = (css[i].scheme.search(/^chrome$|^resource$/) > -1) ? 'jsview-css jsview-italic' : 'jsview-css';

			//get filesize for url if cached
			cacheKey = css[i].spec.replace(/#.*$/,'');
			try {
				entry = cache.openCacheEntry(cacheKey, Components.interfaces.nsICache.ACCESS_READ, false);
				if (filesize) {
					size = ' (' + (entry.dataSize / 1024).toFixed(2) + ' ' + kb + ')';
				} else {
					size = ' (' + jsvutils.addCommas(entry.dataSize) + ' ' + bytes + ')';
				}
			} catch(e) {
				size = '';
			}

			attrs = ['menuitem',
				  ['label',fname + size],
				  ['value',css[i].spec],
				  ['statustext',css[i].spec],
				  ['tooltiptext',css[i].spec],
				  ['oncommand','jsvutils.viewSource(this.value,4);'],
				  ['onclick','jsvutils.promptBehavior(event,this.parentNode,this.value);'],
				  ['onmouseover','jsviewViewSource.showStatusText(this.value);'],
				  ['onmouseout',"jsviewViewSource.showStatusText('" + done + "');"],
				  ['class','menuitem-iconic ' + cname]];
			menu.appendChild(this.makeElement(attrs));
		}
	},

	showStatusText: function(txt) {
		document.getElementById('statusbar-line-col').setAttribute('label',txt);
	}
};
